#include "arbolbinarioordenado.h"
#include "nodobinario.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <iostream.h>

//! Construye el rbol.

/*!
    Construye un rbol binario ordenado con informacin entera.
*/
ArbolBinarioOrdenado::ArbolBinarioOrdenado( )
{
    raiz( NULL );
}

//! Destruye el rbol.

/*!
    Destruye un rbol binario ordenado con informacin entera.
    NO EST IMPLEMENTADO.
 */
ArbolBinarioOrdenado::~ArbolBinarioOrdenado( )
{
    // POR HACER????
    if(raiz()!=NULL)
	{
	_raiz->borrarArbol();
	}
	
}

//! Raiz del arbol.

/*!
    Devuelve un puntero a la raiz del rbol.
 */
NodoBinario *ArbolBinarioOrdenado::raiz( void )
{
    return _raiz;
}

//! Cambia la raiz del rbol

/*!
    Cambia la raiz del rbol.

    \param r Puntero al nodo que ser la nueva raiz.
 */
void ArbolBinarioOrdenado::raiz( NodoBinario *r )
{
    _raiz = r;
    // El padre de la raiz debe ser NULL
    if ( r )
        r->padre( NULL );
}

//! Est el rbol vacio?

/*!
    Indica si el rbol est vacio.

    \returns <tt>true</tt> si el rbol est vacio,
        <tt>false</tt> en caso contrario.
 */
bool ArbolBinarioOrdenado::vacio( void )
{
    if ( !raiz() )
        return true;
    else
        return false;
}

//! Inserta informacin en el rbol.

/*!
    Inserta la informacin <tt>info</tt> en el rbol.

    \param  info Informacin a insertar.
    \returns <tt>true</tt> si se insert la informacin
            <tt>falso</tt> si ya exista.
 */
bool ArbolBinarioOrdenado::inserta( int info )
{
    if ( vacio() ) {
        raiz( new NodoBinario( NULL, info ) );
        return true;
    }

    NodoBinario *p = raiz();

    for (;;) {
        if ( p->informacion() == info )
            return false;

        if ( info < p->informacion() ) {
            if ( ! p->hijoIzquierdo() ) {
                p->hijoIzquierdo( new NodoBinario( p, info ) );
                return true;
            } else {
                p = p->hijoIzquierdo();
            }
        } else {
            if ( ! p->hijoDerecho() ) {
                p->hijoDerecho( new NodoBinario( p, info ) );
                return true;
            } else {
                p = p->hijoDerecho();
            }
        }
    }
}


//! Recorrido para cambiar el arbol...

void ArbolBinarioOrdenado::especularArbol(void)
{
  espejoArbol f;
  if(!vacio())
    _raiz->recorridoPostfijo( &f );
}



//! Recorre el rbol en orden prefijo.

/*!
    Recorre el rbol en orden prefijo aplicando el functor f a cada nodo.
    Al separar el recorrido de la operacin a aplicar evitamos tener
    que codificar una y otra vez el mismo recorrido, tan slo tendremos que
    implementar las diferentes operaciones a realizar.

    Para cada nodo se llamar al mtodo FunctorNodo::aplicar()

    \param f Functor a aplicar a cada nodo.
 */
void ArbolBinarioOrdenado::recorridoPrefijo( FunctorNodo *f )
{
    if ( !vacio() )
        raiz()->recorridoPrefijo( f );
    // POR HACER
}

//! Recorre el rbol en orden infijo.

/*!
    Recorre el rbol en orden infijo aplicando el functor f a cada nodo.
    Al separar el recorrido de la operacin a aplicar evitamos tener
    que codificar una y otra vez el mismo recorrido, tan slo tendremos que
    implementar las diferentes operaciones a realizar.

    Para cada nodo se llamar al mtodo FunctorNodo::aplicar()

    \param f Functor a aplicar a cada nodo.
 */
void ArbolBinarioOrdenado::recorridoInfijo( FunctorNodo *f )
{
    if ( !vacio() )
        raiz()->recorridoInfijo( f );
}

//! Recorre el rbol en orden postfijo.

/*!
    Recorre el rbol en orden postfijo aplicando el functor f a cada nodo.
    Al separar el recorrido de la operacin a aplicar evitamos tener
    que codificar una y otra vez el mismo recorrido, tan slo tendremos que
    implementar las diferentes operaciones a realizar.

    Para cada nodo se llamar al mtodo FunctorNodo::aplicar()

    \param f Functor a aplicar a cada nodo.
 */
void ArbolBinarioOrdenado::recorridoPostfijo( FunctorNodo *f )
{
    if ( !vacio() )
        raiz()->recorridoPostfijo( f );
    // POR HACER
}

   // POR HACER


int ArbolBinarioOrdenado::alturaNodo( NodoBinario *n )
{
int contador=0;
contador=n->altura();
return contador;
}

int ArbolBinarioOrdenado::profundidadNodo( NodoBinario *n )
{
int contador=0;
contador=n->profundidad();
return contador;
}




//! Elimina un nodo.

/*!
    Elimina el nodo apunto por n del rbol.

    \param n Nodo a eliminar.
*/

void ArbolBinarioOrdenado::borrarNodo( NodoBinario *n )
{
  assert( !vacio() && n!=NULL );
  NodoBinario *derecho, *papa; 
  // Precondicin: El rbol no puede estar vacio y n
  // debe ser no nulo
  // para poder borrar un nodo tiene que existir y el arbol no tiene que estar vacio. Si en tiempo de ejecucion lo indica donde a fallado.
  
  // el nodo es una hoja!
  if (n->hijoDerecho()==NULL && n->hijoIzquierdo()==NULL)
    {
      if(n->padre()==NULL) // comprobamos si es la raiz
	{
	  _raiz=NULL; // hay que hacerlo asi.
	}
      else
	{
	  if(n->padre()->hijoDerecho()==n)
	    {
	      n->padre()->hijoDerecho(NULL);
	    }
	  else 
	    {
	      n->padre()->hijoIzquierdo(NULL);
	    }
	  delete n;
	}
    }
  
  // nodo en el hijo izquierdo pero no en el derecho
  
  else if(n->hijoDerecho()==NULL && n->hijoIzquierdo()!=NULL) 
    { 
      if(n->padre()==NULL) // comprobamos si es la raiz
	{
	  raiz(n->hijoIzquierdo());
	}
      else
	{
	  // verificamos que hijo le tenemos que asignar al padre
	  if(n->padre()->hijoDerecho()==n)
	    {
	      n->padre()->hijoDerecho(n->hijoIzquierdo());
	    }
	  else 
	    {
	      n->padre()->hijoIzquierdo(n->hijoIzquierdo());
	    }
	   n->hijoIzquierdo()->padre(n->padre());
	}
      delete n;
    }
  
  // nodo en el hijo Derecho pero no en el Izquierdo
  
  else  if(n->hijoDerecho()!=NULL && n->hijoIzquierdo()==NULL) 
    { 
      if(n->padre()==NULL) // comprobamos si es la raiz
	{
	  raiz(n->hijoDerecho());
	}
     else
       {
	 // verificamos que hijo le tenemos que asignar al padre
	 if(n->padre()->hijoDerecho()==n)
	   {
	     n->padre()->hijoDerecho(n->hijoDerecho());
	   }
	 else 
	   {
	     n->padre()->hijoIzquierdo(n->hijoDerecho());
	   }
	 n->hijoDerecho()->padre(n->padre());
       }
     delete n;
    }
  
  
  // nodos hijo izquierdo y derecho existen.
  
  else 
    {
      papa=n;
      derecho=n->hijoIzquierdo();
      // buscamos el menor de los derechos a partir del nodo izquierdo
      if(derecho->hijoDerecho()!=NULL)
	{
	  while(derecho->hijoDerecho()!=NULL)
	    {
	      papa=derecho; 
	      derecho=derecho->hijoDerecho(); 
	    }
	}
      
      // ahora lo que hacemos es cambiar los nodos....
      
      // copiamos el campo inf al nodo a eliminar.      
      n->informacion(derecho->informacion()); 

      // comprobamos que el papa no sea n.
      if (papa->informacion()!=n->informacion()) 
	{
	  papa->hijoDerecho(derecho->hijoIzquierdo());
	}
      else // sino le asignamos al hijoIzquierdo
	{
	  papa->hijoIzquierdo(derecho->hijoIzquierdo());
	}
      delete derecho;
    }
}



// 

bool ArbolBinarioOrdenado::buscarNodo(int valor)
{
NodoBinario *buscado;
buscado=raiz()->buscar(valor);
if(buscado)
	{
	return true;
	}
else
	{
	return false;
	}
}

int ArbolBinarioOrdenado::hermanoNodo( NodoBinario *n )
{
NodoBinario *temp;
temp=n->hermano();
if(temp!=NULL)
    {
    return temp->informacion();
    }
else
    {
    return -1;
    }
}

// funcion que devuelve el valor del tio, sino tiene tio devuelve -1
int ArbolBinarioOrdenado::tioNodo( NodoBinario *n )
{
NodoBinario *temp;
temp=n->tio();
if(temp!=NULL)
    {
    return temp->informacion();
    }
else
    {
    return -1;
    }
}

// sobrinosNodo. Calcula los sobrinos del nodo actual.
// devuelve entero el valor del nodo sobIzquierdo o sobDerecho.
// si no existiese cualquiera de ellos devuelve -1
void ArbolBinarioOrdenado::sobrinosNodo( NodoBinario  *n, int &sobIzquierdo, int &sobDerecho)
{
NodoBinario *sobIzq, *sobDer;
sobIzq=sobDer=NULL;
n->sobrinos(sobIzq,sobDer);

// asignamos los valores informativos 
if(sobIzq!=NULL)
    {
    sobIzquierdo=sobIzq->informacion();
    }
else
    {
    sobIzquierdo=-1;
    }
if(sobDer!=NULL)
    {
    sobDerecho=sobDer->informacion();
    }
else
    {
    sobDerecho=-1;
    }
}


void ArbolBinarioOrdenado::primosNodo(NodoBinario *n, int &priIzquierdo, int &priDerecho)
{
NodoBinario *priIzq, *priDer;
priIzq=priDer=NULL;
n->primos(priIzq,priDer);
if(priIzq!=NULL)
    {
    priIzquierdo=priIzq->informacion();
    }
else
    {
    priIzquierdo=-1;
    }
if(priDer!=NULL)
    {
    priDerecho=priDer->informacion();
    }
else
    {
    priDerecho=-1;
    }
}

