#include "dibujarnodos.h"
#include "nodobinario.h"

#include <qpainter.h>


DibujarNodos::DibujarNodos(QPainter *p,QRect &r)
{
    _painter=p;
    _recorte=r;
};

void DibujarNodos::aplicar( QRect &cajaNodo, QRect &cajaIzquierda,
            QRect &cajaDerecha, NodoBinario *nodo, bool seleccionado)
{
    if ( _recorte.intersects(cajaNodo) )
        imprimeNodo( _painter, cajaNodo, nodo, seleccionado );

    if ( nodo->hijoIzquierdo() )
        trazaFlecha( _painter, cajaNodo, cajaIzquierda, Qt::red );

    if ( nodo->hijoDerecho() )
        trazaFlecha( _painter, cajaNodo, cajaDerecha, Qt::blue );
}

void DibujarNodos::trazaFlecha( QPainter *p, QRect &cajaPadre,
            QRect &cajaHijo, const QColor &c )
{
    QPoint p1( (cajaPadre.left()+cajaPadre.right())/2, cajaPadre.bottom() );
    QPoint p2( (cajaHijo.left()+cajaHijo.right())/2, cajaHijo.top() );

    p->setPen( c );
    p->drawLine( p1, p2 );
}

void DibujarNodos::imprimeNodo( QPainter *p, QRect &r, NodoBinario *nodo,
    bool seleccionado )
{
    // Centrar el texto en la caja
    p->setPen( Qt::black );
    QString etiqueta( QString::number( nodo->informacion() ) );
    QFontMetrics fm = p->fontMetrics();

    int altoEtiqueta = fm.ascent();
    int anchoEtiqueta = fm.width( etiqueta );

    int px = (r.left()+r.right()-anchoEtiqueta)/2;
    int py = (r.bottom()+r.top()+altoEtiqueta)/2;

    p->drawText( px, py, etiqueta );

    // Si el nodo es el seleccionado lo marcamos ms ancho
    if (seleccionado) {
        QPen lapiz( Qt::black, 3 );
        p->setPen(lapiz);
        p->drawEllipse( r );
    } else {
        p->drawEllipse( r );
    }
}
