#include "nodobinario.h"
#include <stdlib.h>
#include <stdio.h>
#include <algorithm>

//! Construye un nodo binario

/*!
    Construye un nodo binario con un padre e informacin dados.

    \param p Puntero al nodo padre.
    \param info Valor a almacenar.
 */

NodoBinario::NodoBinario( NodoBinario *p, int info )
{
    informacion( info );
    hijoIzquierdo( NULL );
    hijoDerecho( NULL );
    padre( p );
}

//! Destructor

/*!
    Destruye un nodo binario.
 */
NodoBinario::~NodoBinario( )
{
}

//! Asigna el hijo izquierdo

/*!
    Asigna <tt>h</tt> como hijo izquierdo del nodo actual.

    \param h Nuevo hijo izquierdo.
 */
void NodoBinario::hijoIzquierdo( NodoBinario *h )
{
    _hijoIzquierdo = h;
    if ( h )
        h->padre( this );
}

//! Asigna el hijo derecho

/*!
    Asigna <tt>h</tt> como hijo derecho del nodo actual.

    \param h Nuevo hijo derecho.
 */
void NodoBinario::hijoDerecho( NodoBinario *h )
{
    _hijoDerecho = h;
    if ( h )
        h->padre( this );
}

//! Asigna el padre

/*!
    Asigna <tt>p</tt> como padre del nodo actual. Es privada porque no
    tiene sentido cambiar el padre de un nodo desde el exterior (los
    mtodos hijoIzquierdo y derecho ya lo tienen en cuenta).

    \param p Nuevo padre.
 */
void NodoBinario::padre( NodoBinario *p )
{
    _padre = p;
}

//! Devuelve el hijo izquierdo

/*!
    Devuelve el hijo izquierdo del nodo.
 */
NodoBinario *NodoBinario::hijoIzquierdo( void )
{
    return _hijoIzquierdo;
}

//! Devuelve el hijo derecho

/*!
    Devuelve el hijo derecho del nodo.
 */
NodoBinario *NodoBinario::hijoDerecho( void )
{
    return _hijoDerecho;
}

//! Padre del nodo

/*!
    Devuelve el padre del nodo.
 */
NodoBinario *NodoBinario::padre( void )
{
    return _padre;
}

//! Asigna la informacin del nodo

/*!
    Asigna la informacin del nodo.
 */
void NodoBinario::informacion( int x )
{
    _informacion = x;
}

//! Informacin del nodo

/*!
    Devuelve la informacin del nodo.
 */
int NodoBinario::informacion( void )
{
    return _informacion;
}

//! Es este nodo una hoja?

/*!
    Determina si el nodo es una hoja.

    \returns <tt>true</tt> si el nodo es una hoja,
        <tt>false</tt> en caso contrario.
 */
bool NodoBinario::esHoja( void )
{
    if ( !hijoIzquierdo() && !hijoDerecho() )
        return true;
    else
        return false;
}

//! Es este nodo la raiz del rbol?

/*!
    Determina si el nodo es la raiz de su rbol.

    \returns <tt>true</tt> si el nodo es la raiz,
        <tt>false</tt> en caso contrario.
 */
bool NodoBinario::esRaiz( void )
{
    if ( !padre() )
        return true;
    else
        return false;
}

//! Profundidad

/*!
    Determina la profundidad del nodo.

    \returns Profundidad del nodo.
 */
int NodoBinario::profundidad( void )
{
int contador;
 NodoBinario *n;
contador=0;
 n=this;
while(!n->esRaiz())
    {
    n=n->padre();
    contador++;
    }

    // POR HACER
    return contador;
}

//! Altura

/*!
    Determina la altura del nodo.

    \returns Altura del nodo.
 */

int NodoBinario::altura( void )
{
    // POR HACER
if(this==NULL)
    {
    return 0;
    }
else
    {
    if(this->hijoIzquierdo()==NULL && this->hijoDerecho()==NULL)
	{
	return 0;
	}
    else
	{
	return 1+max(this->hijoIzquierdo()->altura(),this->hijoDerecho()->altura());
	}
    }
}


//! Recorre el rbol en orden prefijo.

/*!
    Recorre el subrbol con raiz en el nodo actual en orden prefijo, aplicando
    el functor f a cada nodo. Al separar el recorrido de la operacin a
    aplicar evitamos tener que codificar una y otra vez el mismo recorrido,
    tan slo tendremos que implementar las diferentes operaciones a
    realizar.

    Para cada nodo se llamar al mtodo FunctorNodo::aplicar()

    \param f Functor a aplicar a cada nodo.
 */
void NodoBinario::recorridoPrefijo( FunctorNodo *f )
{

    f->aplicar( this );

    if ( hijoIzquierdo() )
        hijoIzquierdo()->recorridoPrefijo( f );


    if ( hijoDerecho() )
        hijoDerecho()->recorridoPrefijo( f );

    // POR HACER
}

/*!
    Recorre el subrbol con raiz en el nodo actual en orden infijo, aplicando
    el functor f a cada nodo. Al separar el recorrido de la operacin a
    aplicar evitamos tener que codificar una y otra vez el mismo recorrido,
    tan slo tendremos que implementar las diferentes operaciones a
    realizar.

    Para cada nodo se llamar al mtodo FunctorNodo::aplicar()

    \param f Functor a aplicar a cada nodo.
 */
void NodoBinario::recorridoInfijo( FunctorNodo *f )
{
    if ( hijoIzquierdo() )
        hijoIzquierdo()->recorridoInfijo( f );

    f->aplicar( this );

    if ( hijoDerecho() )
        hijoDerecho()->recorridoInfijo( f );
}

//! Recorre el rbol en orden postfijo.

/*!
    Recorre el subrbol con raiz en el nodo actual en orden prefijo, aplicando
    el functor f a cada nodo. Al separar el recorrido de la operacin a
    aplicar evitamos tener que codificar una y otra vez el mismo recorrido,
    tan slo tendremos que implementar las diferentes operaciones a
    realizar.

    Para cada nodo se llamar al mtodo FunctorNodo::aplicar()

    \param f Functor a aplicar a cada nodo.
 */
void NodoBinario::recorridoPostfijo( FunctorNodo *f )
{

    if ( hijoIzquierdo() )
        hijoIzquierdo()->recorridoPostfijo( f );
    if ( hijoDerecho() )
        hijoDerecho()->recorridoPostfijo( f );
    f->aplicar( this );

}

// funcion para destruir el arbol de manera recursiva
void NodoBinario::borrarArbol( void )
{
if(this->hijoIzquierdo()!=NULL)  // primero todos los nodos izquierdos.
    {
    this->hijoIzquierdo()->borrarArbol();
    }
else if(this->hijoDerecho()!=NULL) // luego los derechos
    {
    this->hijoDerecho()->borrarArbol();
    }
delete this; // borramos el nodo actual
}


// Funciones relativas a Busqueda de nodos.

/* Busca un nodo dado por el "valor" si lo encuentra lo devuelve como
puntero, sino devuelve NULL */
NodoBinario *NodoBinario::buscar (int valor)
{
  if(this->informacion() > valor)
    {
      if(this->hijoIzquierdo()!=NULL)
        {
          return this->hijoIzquierdo()->buscar(valor);
        }
      else
        {
          return NULL;
        }
    }
  else if(this->informacion() < valor)
    {
      if(this->hijoDerecho()!=NULL)
        {
          return this->hijoDerecho()->buscar(valor);
        }
      else
        {
          return NULL;
        }
    }
  else
    {
      return this;
    }
}


// hermano. Calcula el nodo hermano.
NodoBinario *NodoBinario::hermano( void )
{
  if(this->padre()==NULL) // comprobamos que no es la raiz.
    {
    return NULL;
    }
else
    {
      // calculamos el hermano, viendo si el nodo actual es
      // el hijoDerecho del padre.
      if(this->padre()->hijoDerecho()==this) 
	{
	return this->padre()->hijoIzquierdo();
	}
    else
	{
	return this->padre()->hijoDerecho();
	}
    }
}


// tio. funcion que devuelve el nodo tio
NodoBinario *NodoBinario::tio( void )
{
  if(this->padre()!=NULL) // vemos si tiene padre,
    {
      return this->padre()->hermano();
    }
  else
    {
      return NULL;
    }
}


// sobrinos. Funcion que calcula los sobrinos de un nodo.
void NodoBinario::sobrinos(NodoBinario * &sobIzquierdo, NodoBinario * &sobDerecho )
{
  if(this->hermano()==NULL) // verificamos que no tiene hermano el nodo.
    {
      sobIzquierdo=NULL;
      sobDerecho=NULL;
    }
  else // si tiene hermano pues calculamos los hijos del hermano.
    {
      // ahora calculamos los sobrinos
      sobIzquierdo=this->hermano()->hijoIzquierdo();
      sobDerecho=this->hermano()->hijoDerecho();
    }
}

// primos, funcion que calcula los primos de un nodo dado.
void NodoBinario::primos(NodoBinario * &priIzquierdo, NodoBinario * &priDerecho )
{
  if(this->padre()!=NULL) // si no es raiz
    {
      // calculamos los sobrinos del padre, que sera el tio.
      this->padre()->sobrinos(priIzquierdo,priDerecho);
    }
  else // sino no tiene primos.
    {
      priIzquierdo=NULL;
      priDerecho=NULL;
    }
}
