#ifndef __NODOBINARIO_H__
#define __NODOBINARIO_H__

#include "functornodo.h"

//! Clase NodoBinario nodo de un rbol binario

/*!
    Esta clase representa un nodo de un rbol binario
    ordenado que almacena informacin entera.

    La principal diferencia con lo explicado en teora es que
    cada nodo incluye un tercer puntero, que apunta a su nodo padre
    (o <tt>NULL</tt> en el caso de la raiz). Esto permite simplificar
    gran cantidad de operaciones como, por ejemplo, el borrado.
 */

class NodoBinario {
    friend class ArbolBinarioOrdenado;
    friend class espejoArbol; // asi para que pueda acceder a las funciones privadas

    public:
        NodoBinario( NodoBinario *p, int info );
        ~NodoBinario( );

        void informacion( int x );
        int informacion( void );

        NodoBinario *hijoIzquierdo( void );
        NodoBinario *hijoDerecho( void );
        NodoBinario *padre( void );
        
//
	NodoBinario *buscar (int valor);
        NodoBinario *hermano(void);
	NodoBinario *tio(void);
	void sobrinos(NodoBinario  * &sobIzquierdo, NodoBinario * &sobDerecho);
	void primos(NodoBinario * &priIzquierdo, NodoBinario * &priDerecho);
//	
        bool esHoja( void );
        bool esRaiz( void );

        void recorridoPrefijo( FunctorNodo *f );
        void recorridoInfijo( FunctorNodo *f );
        void recorridoPostfijo( FunctorNodo *f );


	int altura( void );
	int profundidad( void );
// 
	void borrarArbol( void );
	

    private:
        void hijoIzquierdo( NodoBinario *h);
        void hijoDerecho( NodoBinario *h);
        void padre( NodoBinario *p );

        //! Informacin almacenada

        /*!
            Esta es la informacin que almacena el nodo.
            Para hacerlo mejor, esta clase podra ser un
            <i>template</i>. De esta forma, _informacion
            tendra como tipo el parmetro del template.
         */
        int _informacion;

        //! Puntero al hijo izquierdo

        /*!
             El puntero al hijo izquierdo. NULL si no lo tiene.
         */
        NodoBinario *_hijoIzquierdo;

        //! Puntero al hijo derecho

        /*!
             El puntero al hijo derecho. NULL si no lo tiene.
         */
        NodoBinario *_hijoDerecho;

        //! Puntero al nodo padre

        /*!
             El puntero al nodo padre. NULL si no lo tiene.
         */
        NodoBinario *_padre;
};

#endif

