#include "practica.h"
#include "insertarlista.h"
#include "arbolbinarioordenado.h"
#include "visorarbol.h"
#include "functornodo.h"
#include "nodobinario.h"

#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qinputdialog.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qlistbox.h>
//

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <time.h>
#include <malloc.h>

// Declara la variable UCOsmall_xpm como pixmap
#include "ucosmall.xpm"

//! Crear un widget Practica


/*!
    Crea un widget Practica, que es la ventana principal de nuestra aplicacin.

    \param parent Widget padre.
    \param name Nombre del widget.
    \param f Opciones del widget.
 */
Practica::Practica ( QWidget * parent = 0, const char * name = 0,
                     WFlags f = WType_TopLevel ):QMainWindow( parent, name, f )
{

    // Creamos el interfaz de usuario
    crearInterfaz();

    // Creamos el rbol (nuestro documento)
	_arbol = new ArbolBinarioOrdenado();

    // La vista del rbol
	_vista = new VisorArbol( _arbol, this );

    // Conectamos nuestra seal de modificacin de rbol
    // al slot de la vista. De esta forma la vista se
    // actualizar cada vez que modifiquemos el  rbol.
	connect( this, SIGNAL( arbolCambiado( ArbolBinarioOrdenado * ) ),
        _vista, SLOT( arbolModificado( ArbolBinarioOrdenado * ) ) );

    // Conectamos la seal nodoSeleccionado() de la vista
    // con nuestro slot nodoSeleccionado() de esta forma
    // siempre sabemos que nodo selecciona el usuario
    connect( _vista, SIGNAL( nodoSeleccionado( NodoBinario * ) ),
        this, SLOT( nodoSeleccionado( NodoBinario * ) ) );

    // La vista es el widget central de la ventana
	setCentralWidget( _vista );

    // Inicialmente no hay ningn nodo seleccionado
    _nodoActual = NULL;

    // No esta inicializado inicialmente el arbol especular
    _especular=false;
}

//! Destruye un objeto Practica

/*!
    Destruye la aplicacin.
 */
Practica::~Practica()
{
    // Borramos el rbol
	if (_arbol)
		delete _arbol;
    // Los widgets hijo se liberan slos
}

//! El usuario ha seleccionado un nodo

/*!
    Slot que se activa cuando el usuario ha seleccionado un nodo
    en el rbol.

    \param n Nodo seleccionado.
 */
void Practica::nodoSeleccionado( NodoBinario *n )
{
  if (n) // Nodo Seleccionado HABILITAR
    {
      cout << "\nSEleccionado" << endl;
      menuNodo->setItemEnabled(alturaMenu,TRUE);
      menuNodo->setItemEnabled(profundidadMenu,TRUE);
      menuBusquedas->setItemEnabled(hermanoBusquedas,TRUE);
      menuBusquedas->setItemEnabled(tioBusquedas,TRUE);
      menuBusquedas->setItemEnabled(sobrinosBusquedas,TRUE);
      menuBusquedas->setItemEnabled(primosBusquedas,TRUE);
      if(_especular==true)
 	{
	// Si es especular deshabilitamos, borrar,insertar y buscar.
	  menuNodo->setItemEnabled(borrarMenu,FALSE);
	  menuNodo->setItemEnabled(nuevonodoMenu,FALSE);
	  menuBusquedas->setItemEnabled(buscarBusquedas,FALSE);
	}
      else
	{
	// sino es especular habilitamos...
	  menuNodo->setItemEnabled(nuevonodoMenu,TRUE);
	  menuBusquedas->setItemEnabled(buscarBusquedas,TRUE);
	  menuNodo->setItemEnabled(borrarMenu,TRUE);
	}
    }
  else   // DEShabilitar, no hay nodo seleccionado
    {
      if(_especular==true)
 	{
	  menuNodo->setItemEnabled(nuevonodoMenu,FALSE);
	  menuBusquedas->setItemEnabled(buscarBusquedas,FALSE);
	}
      else
	{
	  menuBusquedas->setItemEnabled(buscarBusquedas,TRUE);
	  menuNodo->setItemEnabled(nuevonodoMenu,TRUE);
	}
      menuNodo->setItemEnabled(borrarMenu,FALSE);
      menuNodo->setItemEnabled(alturaMenu,FALSE);
      menuNodo->setItemEnabled(profundidadMenu,FALSE);
      menuBusquedas->setItemEnabled(hermanoBusquedas,FALSE);
      menuBusquedas->setItemEnabled(tioBusquedas,FALSE);
      menuBusquedas->setItemEnabled(sobrinosBusquedas,FALSE);
      menuBusquedas->setItemEnabled(primosBusquedas,FALSE);
      
      
    }
  _nodoActual=n;
  
  // Interesante:
  //
    // Si n==NULL deshabilitar en el menu las operaciones de nodo
  // excepto "Nuevo nodo"
  // Si n!=NULL habilitarlas
  //
  // PISTA: Hasta ahora las opciones son locales a
  // Practica::crearInterfaz(), para hacer esto
  // tendrn que ser miembros de la clase y
  // acceder desde aqui.
}

//! Inserta un nuevo nodo

/*!
    Este slot se activa para insertar un nuevo nodo.
 */
void Practica::nuevoNodo( void )
{
	bool ok=true;

    // Pedimos un entero al usuario
	QString str=QInputDialog::getText( "Aadir nodo",
        "Introduzca un valor entero",
        QLineEdit::Normal, QString::null, &ok, this );

	if ( ok && !str.isEmpty() ) {
		int numero = str.toInt( &ok );
		if (! ok ) {
			QMessageBox::critical(this, appName,
			    "Debe introducir un nmero entero");
		} else {
			if (! _arbol->inserta( numero ) ) {
				QMessageBox::information(this, appName,
				    "El nmero ya perteneca al rbol" );
			} else {
                // Notificamos un cambio en el rbol
				emit( arbolCambiado( _arbol ) );
			}
		}
	}
}

//! Borra el nodo seleccionado

/*!
    Este slot se activa para borrar el nodo seleccionado.
 */
void Practica::borrarNodo( void )
{
    if ( !_nodoActual ) {
        QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );
    } else {
//        QMessageBox::critical( this, appName, "Opcion no implementada");

        _arbol->borrarNodo( _nodoActual );

        // Interesante: Conservar la seleccin del nodo actual
        _nodoActual = NULL;
        emit( arbolCambiado( _arbol ) );
    }
}

//! Altura del nodo seleccionado

/*!
    Este slot se activa para calcular la altura del nodo seleccionado. NO EST
    IMPLEMENTADO.
  */

void Practica::alturaNodo( void )
{
// Mi pedazo mensaje...
QString Mensaje;
char *caracter;
caracter=(char *)malloc(100*sizeof(char));

int valor;
    if ( !_nodoActual ) 
      {
        QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );
      } 
    else 
      {

// esto es lo que hay que implememntar!!!

valor=_arbol->alturaNodo ( _nodoActual);

// Ahora copiamos el valor...
sprintf(caracter,"%d",valor);
Mensaje = "La altura es " + QString(caracter);

        QMessageBox::information( this, "altura", Mensaje );


//         QMessageBox::critical( this, appName, "Altura no implementada");
      }
}

//! Profundidad del nodo seleccionado

/*!
    Este slot se activa para calcular la profundidad del nodo seleccionado. NO
    EST IMPLEMENTADO.
  */
void Practica::profundidadNodo( void )
{
QString Mensaje;
char *caracter;
caracter=(char *)malloc(100*sizeof(char));
int valor;


    if ( !_nodoActual ) {
        QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );

    } else {
	    valor=_arbol->profundidadNodo ( _nodoActual);

	    // Ahora copiamos el valor...
	    sprintf(caracter,"%d",valor);
	    Mensaje = "La profundidad es " + QString(caracter);

    	    QMessageBox::information( this, "profundidad", Mensaje );

//        QMessageBox::critical( this, appName, "Profundidad no implementada" );
    }
}

//! Recorrido prefijo

/*!
    Este slot se activa para recorrer, de forma prefija, el rbol.
 */
void Practica::recorrerPrefijo( void )
{
    recorrerArbol( prefijo );
}

//! Recorrido infijo

/*!
    Este slot se activa para recorrer, de forma infija, el rbol.
 */
void Practica::recorrerInfijo( void )
{
    recorrerArbol( infijo );
}

//! Recorrido postfijo

/*!
    Este slot se activa para recorrer, de forma postfija, el rbol.
 */
void Practica::recorrerPostfijo( void )
{
    recorrerArbol( postfijo );
}

//! Recorridos del rbol

/*!
    Este mtodo implementa los diferentes recorridos del rbol.
 */
void Practica::recorrerArbol( TipoRecorrido t )
{
    // Creamos una lista
    QListBox *lista = new QListBox( NULL, NULL, Qt::WDestructiveClose );

    // Creamos un functor que inserte el campo informativo de
    // cada nodo en la lista
    InsertarLista functor( lista );

    // Segn el tipo de recorrido
    switch (t) {
        case prefijo:
            lista->setCaption( "Recorrido prefijo" );
	        _arbol->recorridoPrefijo( &functor );
            break;
        case infijo:
            lista->setCaption( "Recorrido infijo" );
	        _arbol->recorridoInfijo( &functor );
            break;
        case postfijo:
            lista->setCaption( "Recorrido postfijo" );
	        _arbol->recorridoPostfijo( &functor );
            break;
    }

    // Mostramos la lista
	lista->show();
}

//! rbol aleatorio

/*!
    Inserta un nmero de nodos dado por el usuario con valor aleatorio
    entre 0 y 1000.
 */
void Practica::arbolAleatorio( void )
{
	bool ok=true;

	int nodos=QInputDialog::getInteger( "Nmero de nodos",
	    "Introduzca un nmero de nodos", 1, 1, 1000, 1, &ok, this );

	if ( ok ) {
		srand( time(NULL) );
		for ( int i=0; i < nodos; ) {
			int numero=rand()%1000;
			if (_arbol->inserta( numero ) )
				i++;
		}
		emit( arbolCambiado( _arbol ) );
	}
}

void Practica::arbolPredefinido( void )
{
  _arbol->inserta(50);
  _arbol->inserta(30);
  _arbol->inserta(75);
  _arbol->inserta(20);
  _arbol->inserta(45);
  _arbol->inserta(10);
  _arbol->inserta(65);
  _arbol->inserta(80);
  _arbol->inserta(90);
  _arbol->inserta(100);
  emit( arbolCambiado( _arbol ) );
}



// Arbol Especular

void Practica::Especular( void )
{
  _nodoActual=NULL;
  if(_arbol)
    {
      if(_especular==false)
	{
	  _especular=true;	
	}
      else
	{
	  _especular=false;
	}
      _arbol->especularArbol();
      emit(arbolCambiado( _arbol ));
      //	QMessageBox::critical( this, appName, "Para protejer la integridad de la estructura arbol se volvera a aplicar cuando de lde a OK");
      //   _arbol->especularArbol();
    }
  emit(arbolCambiado( _arbol ));
}



// Funciones de Busqueda

void Practica::buscarNodo( void )
{
  bool resultado, aceptar=true;
  int valor =QInputDialog::getInteger ( "Buscar un nodo",
					"Introduce el valor del nodo a buscar", 1, 1, 1000, 1, &aceptar, this);
  resultado=_arbol->buscarNodo( valor );
  if(resultado==false)
    {
      QMessageBox::critical( this, appName, "El nodo no existe" );
    }
  else
    {
      QMessageBox::critical( this, appName, "El nodo existe" );
    }
}

void Practica::hermanoNodo( void )
{
  QString Mensaje;
  char *caracter;
  caracter=(char *)malloc(100*sizeof(char));
  int valor;
  
  //
  if ( !_nodoActual ) 
    {
      QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );
    } 
  else 
    {
      
      valor=_arbol->hermanoNodo( _nodoActual );
      if(valor==-1)
	{
	  Mensaje = "No tiene nodo hermano ";
	}
      else
	{
	  sprintf(caracter,"%d",valor);
	  Mensaje = "El hermano es " + QString(caracter);
	}
      QMessageBox::information( this, "Hermano", Mensaje );
    }
}

void Practica::tioNodo( void )
{
  QString Mensaje;
  char *caracter;
  caracter=(char *)malloc(100*sizeof(char));
  int valor;
  
  //
  if ( !_nodoActual ) 
    {
      QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );
    } 
  else 
    {
      
      valor=_arbol->tioNodo( _nodoActual );
      if(valor==-1)
	{
	  Mensaje = "No tiene tio";
	}
      else
	{
	  sprintf(caracter,"%d",valor);
	  Mensaje = "El tio es " + QString(caracter);
	}
      QMessageBox::information( this, "Tio", Mensaje );
    }
}

void Practica::primosNodo( void )
{
  QString Mensaje;
  char *caracter, *cadena;
  cadena=(char *)malloc(100*sizeof(char));
  caracter=(char *)malloc(20*sizeof(char));
  
  int priIzquierdo, priDerecho;
  
  if ( !_nodoActual ) 
    {
      QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );
    } 
  
  else 
    {
      _arbol->primosNodo( _nodoActual, priIzquierdo, priDerecho );
      
      if(priIzquierdo==-1)
	{
	  strcpy(cadena,"No tiene primo izquierdo y");
	}
      
      else
	{
	  sprintf(caracter,"%d",priIzquierdo);
	  //	  Mensaje = "El  " + QString(caracter);
	  strcpy(cadena,"El primo Izquierdo es ");
	  strcat(cadena,caracter);
	  strcat(cadena," y ");
	}
      if(priDerecho==-1)
	{
	  strcat(cadena," no tiene primo derecho");
	}
      else
	{
	  sprintf(caracter,"%d",priDerecho);
	  strcat(cadena," el primo primo derecho es ");
	  strcat(cadena,caracter);
	}
      Mensaje = QString(cadena);
      
      QMessageBox::information( this, "Primos", Mensaje );
      // depurar

    }
}

void Practica::sobrinosNodo( void )
{
  QString Mensaje;
  char *caracter, *cadena;
  cadena=(char *)malloc(100*sizeof(char));
  caracter=(char *)malloc(20*sizeof(char));
  
  int sobIzquierdo, sobDerecho;
  
  if ( !_nodoActual ) 
    {
      QMessageBox::critical( this, appName, "Debe seleccionar un nodo" );
    } 
  
  else 
    {
      _arbol->sobrinosNodo( _nodoActual, sobIzquierdo, sobDerecho );
      
      if(sobIzquierdo==-1)
	{
	  strcpy(cadena,"No tiene sobrino izquierdo y");
	}
      
      else
	{
	  sprintf(caracter,"%d",sobIzquierdo);
	  //	  Mensaje = "El  " + QString(caracter);
	  strcpy(cadena,"El sobrino Izquierdo es ");
	  strcat(cadena,caracter);
	  strcat(cadena," y ");
	}
      if(sobDerecho==-1)
	{
	  strcat(cadena," no tiene sobrino derecho");
	}
      else
	{
	  sprintf(caracter,"%d",sobDerecho);
	  //	  Mensaje = "El  " + QString(caracter);
	  strcat(cadena,"el primo sobrino derecho es ");
	  strcat(cadena,caracter);
	}
      Mensaje = QString(cadena);
      
      QMessageBox::information( this, "Sobrinos", Mensaje );

    }
}





//! Acerca de

/*!
  Este slot se activa para mostrar el cuadro de dilogo "Acerca de..."
*/
void Practica::about( void )
{
  QMessageBox::about(this, tr( "Acerca de Prctica de rboles" ),
		     tr( "Esqueleto para la <b>prctica de rboles</b><br>"
			 "Estructuras de datos y de la Informacin<br>"
			 "Curso 2001/2002<br>"
			 "Creado por Manuel Prieto Villegas cc1prvim@uco.es<br>"
			 "Grupo de investigacin Aplicaciones de la Visin Artificial<br>"
			 "Departamento de Informtica y Anlisis Numrico<br>"
			 "Universidad de Crdoba<br>" ) );
}

//! Crea el interfaz de usuario

/*!
  Este mtodo crea el interfaz de usuario de la aplicacin.
*/
void Practica::crearInterfaz( void )
{
  // Ponemos el caption y el icono de la ventana
  setCaption( "Prctica de rboles" );
  setIcon( QPixmap( UCOsmall_xpm ) );
	
  // Creamos el men archivo
  QPopupMenu *menuArchivo = new QPopupMenu( this );
  menuArchivo->insertItem( "Salir", this, SLOT(close()) );
  menuBar()->insertItem( "Archivo", menuArchivo );
  
  // El men rbol
  QPopupMenu *menuArbol = new QPopupMenu( this );
  menuArbol->insertItem( "Aleatorio", this, SLOT(arbolAleatorio()) );
  menuArbol->insertItem( "Predefinido", this, SLOT(arbolPredefinido()) );
  menuArbol->insertItem( "Especular", this, SLOT(Especular()) );
  menuBar()->insertItem( "Arbol", menuArbol );
  
  // Ahora el men nodo
  //	QPopupMenu * // Se tienen que deshabilitar para poder
  // usar lo de habilitar deshabilitarnodo, sino segfault
  menuNodo = new QPopupMenu( this );
  nuevonodoMenu=menuNodo->insertItem( "Nuevo", this, SLOT(nuevoNodo()) );
  borrarMenu=menuNodo->insertItem( "Borrar", this, SLOT(borrarNodo()) );
  alturaMenu=menuNodo->insertItem( "Altura", this, SLOT(alturaNodo()) );
  profundidadMenu=menuNodo->insertItem( "Profundidad", this, SLOT(profundidadNodo()) );
  menuBar()->insertItem( "Nodo", menuNodo );
  
  // Menu de Busquedas
  // idem que el caso anterior
  //	QPopupMenu *
  menuBusquedas = new QPopupMenu( this );
  buscarBusquedas=menuBusquedas->insertItem( "Buscar Nodo", this, SLOT(buscarNodo()));
  hermanoBusquedas=menuBusquedas->insertItem( "Hermano", this, SLOT(hermanoNodo()) );
  tioBusquedas=menuBusquedas->insertItem( "Tio", this, SLOT(tioNodo()) );
  primosBusquedas=menuBusquedas->insertItem( "Primos", this, SLOT(primosNodo()) );
  sobrinosBusquedas=menuBusquedas->insertItem( "Sobrinos", this, SLOT(sobrinosNodo()) );
  menuBar()->insertItem( "Busquedas", menuBusquedas );

  
  
  // El men de recorridos
  QPopupMenu *menuRecorrido= new QPopupMenu( this );
  menuRecorrido->insertItem("Prefijo", this,
			    SLOT( recorrerPrefijo() ) );
  menuRecorrido->insertItem("Infijo", this,
			    SLOT( recorrerInfijo() ) );
  menuRecorrido->insertItem("Postfijo", this,
			    SLOT( recorrerPostfijo() ) );
  menuBar()->insertItem( "Recorrido", menuRecorrido );
  
  // El men de ayuda
  QPopupMenu *menuAyuda = new QPopupMenu( this );
  menuAyuda->insertItem( "Acerca de...", this, SLOT(about()) );
  menuBar()->insertItem( "Ayuda", menuAyuda );
}
