#ifndef __PRACTICA_H__
#define __PRACTICA_H__

#include <qmainwindow.h>


class ArbolBinarioOrdenado;
class NodoBinario;
class VisorArbol;
class QListBox;
class FunctorNodo;

//! Clase principal de la prctica

/*!
    Esta es la ventana principal de la aplicacin. Contiene una barra de
    menu con las opciones ms comunes (que tendris que ampliar segn
    avance la prctica) y un rbol binario ordenado que se representa
    mediante un widget de la clase que hemos denominado VisorArbol.

    La comunicacin entre las seales y slots de las clases Practica y
    VisorArbol permiten que se mantenga la vista del rbol actualizada.
*/
class Practica:public QMainWindow {
    Q_OBJECT

    public:
        Practica ( QWidget * parent = 0, const char * name = 0,
            WFlags f = WType_TopLevel );
        virtual ~Practica();

    signals:
        //! Se emite esta seal cada vez que se cambia el rbol

        /*!
            El widget emite esta seal cada vez que se modifica
            el rbol. De esta forma, se sincroniza con la vista.
         */
        void arbolCambiado( ArbolBinarioOrdenado * );

    protected:
        enum TipoRecorrido { prefijo, infijo, postfijo };
        void recorrerArbol( TipoRecorrido );

    protected slots:
        void nuevoNodo( void );
        void borrarNodo( void );
        void alturaNodo( void );
//
	void buscarNodo( void );
	void hermanoNodo( void );
	void tioNodo( void );
	void primosNodo( void );
	void sobrinosNodo( void );
//
        void profundidadNodo( void );
        void nodoSeleccionado( NodoBinario *n );
        void Especular( void );

        void arbolAleatorio( void );
        void arbolPredefinido( void );


        void about( void );
        void recorrerPrefijo( void );
        void recorrerInfijo( void );
        void recorrerPostfijo( void );

    private:
        //! El rbol con el que trabaja la aplicacin

        /*!
            La aplicacin trabaja slo con un rbol, como
            construir una aplicacin MDI (interfaz de mltiples
            documentos) lo veremos en la prctica de grafos.
         */
        ArbolBinarioOrdenado *_arbol;

        //! La "vista" del rbol

        /*!
            Este widget hijo (que es el widget central de la
            aplicacin) es el encargado de representar el rbol.
         */
        VisorArbol *_vista;

        //! El nodo seleccionado por el usuario

        /*!
            El nodo que selecciona el usuario lo almacenamos aqu.
            Esta informacin se obtiene de la seal
            VisorArbol::nodoSeleccionado().
         */
        NodoBinario *_nodoActual;

        //! El nombre de la aplicacin

        /*!
            El nombre de la aplicacin. Slo lo usamos como
            caption de las ventanas hijas.
         */
        static const char appName[]="Prctica de rboles";
        void crearInterfaz( void );
 
	// Para habilitar deshabilitar el menu
        int borrarMenu, alturaMenu, profundidadMenu, nuevonodoMenu, buscarBusquedas ,hermanoBusquedas, tioBusquedas, primosBusquedas, sobrinosBusquedas;
        QPopupMenu *menuNodo, *menuBusquedas; // tiene que ser asi!

	// variable para controlar el arbol Especular
	bool _especular;
};

#endif
